@@ -5,11 +5,13 @@ if PANDAS:
     import pandas as pd
 from datetime import datetime
 import simplejson as JSON
-import re, io
+import re, io, math
 from collections import namedtuple
 from multiprocessing import Pool
 from tqdm import tqdm, tqdm_notebook
 from time import sleep
+import cbor2
+from shapely.geometry import Point
 
 from queue import Queue
 from threading import Thread
@@ -18,19 +20,19 @@ Update = namedtuple('Update', ['id', 'start', 'end', 'data'])
 
 def index_data(args):
     db, node, index, v = args
-    if index == 'event':
-        return
     counter = 0
     while counter < 10:
         try:
-            resp = db._post('nodes', node, 'types', index, json=v)
+            resp = db._post('nodes', node, 'types', index,
+                    json=cbor2.dumps(v), headers={'Content-Type': 'application/cbor'}, raw=True)
         except:
+            print(v[:10])
             counter += 1
             sleep(.1)
         else:
+            if resp.status_code > 204:
+                raise Exception(f"failed on row {i}, column {k}")
             break
-    if resp.status_code > 204:
-        raise Exception(f"failed on row {i}, column {k}")
 
 class WQueue(Queue):
     def write(self, data):
@@ -274,6 +276,14 @@ class Database(API):
                 tmap[cname] = 'datetime'
             elif df[cname].dtype == np.dtype('timedelta64[ns]'):
                 tmap[cname] = 'timedelta'
+            elif type(df[cname][0]) == date:
+                tmap[cname] = 'date'
+            elif type(df[cname][0]) == time:
+                tmap[cname] = 'time'
+            elif type(df[cname][0]) == Point and df[cname][0].has_z:
+                tmap[cname] = 'point3'
+            elif type(df[cname][0]) == Point:
+                tmap[cname] = 'point'
             else:
                 tmap[cname] = 'text'
 
@@ -282,6 +292,7 @@ class Database(API):
             dmap[cname] = []
 
         origin = self.origin
+        print(tmap)
         for i, row in tqdm(df.iterrows(), total=len(df), unit='values', unit_scale=len(df.columns) - 1):
             if origin is not None:
                 ts = (row['start'] - origin).delta
@@ -292,24 +303,38 @@ class Database(API):
                     dur = (row['end'] - row['start']).delta
                 elif origin is not None:
                     dur = (df['start'].iloc[i+1] - row['start']).delta
+                elif 'end' in row:
+                    dur = df['end'] - df['start']
                 else:
-                    raise Exception("origin should have been detected.")
-                #elif 'end' in row:
-                #    dur = df['end'] - df['start']
-                #else:
-                #    dur = (df['start'].iloc[i+1] - df['start']) / np.timedelta64(1, 'ns')
+                    dur = (df['start'].iloc[i+1] - df['start']) // np.timedelta64(1, 'ns')
             except IndexError:
                 dur = 1
             else:
                 if dur <= 0: continue
                 for col, val in dict(row).items():
                     if col == 'start':
-                        dmap[col].append({'ts': ts, 'duration': dur})
+                        dmap[col].append((ts, dur))
+                    elif type(val) == float and math.isnan(val): # skip nans
+                        pass
+                    elif val is pd.NaT or val is None:
+                        pass
+                    elif tmap[col] == 'point3':
+                        dmap[col].append((ts, dur, (val.x, val.y, val.z)))
+                    elif tmap[col] == 'point':
+                        dmap[col].append((ts, dur, (val.x, val.y)))
+                    elif tmap[col] == 'date':
+                        dmap[col].append((ts, dur, val.isoformat()))
+                    elif tmap[col] == 'time':
+                        dmap[col].append((ts, dur, val.isoformat()))
+                    elif tmap[col] == 'datetime':
+                        dmap[col].append((ts, dur, iso8601(val)))
+                    elif tmap[col] == 'timedelta':
+                        dmap[col].append((ts, dur, val // np.timedelta64(1, 'ns')))
                     else:
-                        dmap[col].append({'ts': ts, 'duration': dur, 'value': val})
+                        dmap[col].append((ts, dur, val))
 
-            if len(dmap['start']) >= 1024:
-                with Pool(processes=32) as pool:
+            if len(dmap['start']) >= 4096:
+                with Pool(processes=16) as pool:
                     pool.map(index_data, [(self, cmap[k], tmap[k], dmap[k]) for k, v in dmap.items()])
                 for k, v in dmap.items():
                     dmap[k] = []
@@ -437,8 +462,7 @@ class Stream(API):
         return StreamStats(self)
 
     def __len__(self):
-        raise NotImplemented
-        return StreamStats(self, vtype="event", ro=True).count
+        return StreamStats(self, vtype=self.type, ro=True).count
     
 class StreamStats(object):
     def __init__(self, parent, start=None, end=None, origin=None, vtype=None, ro=False):
@@ -484,7 +508,7 @@ class StreamStats(object):
         #end = self._end or origin + td64(2 ** 63 - 1)
         try:
             return self._parent._parent._parent(f"""
-                {stat}({self._parent})
+                {stat}({"when " if self._type == 'event' else ''}{self._parent})
                     when interval({iso8601(start)}, {iso8601(end)})
                     origin {iso8601(origin)}
             """)[::1][0]['value']
@@ -542,6 +566,8 @@ class StreamData(API):
         API.__init__(self, parent._credentials, *parent._prefix, "types", index)
 
     def resample(self, period, aggregator=None):
+        if self._type == 'event':
+            aggregator = "count"
         return StreamData(self._parent, self._type, self._df, (period, aggregator), self._rolling)
 
     def rolling(self, period):
@@ -559,7 +585,7 @@ class StreamData(API):
             else:
                 rs = f'{self._parent!s} when frequency({self._resample[0]}) {r}'
         else:
-            rs = f'{self._parent!s} {r}'
+            rs = f'{"when " if self._type == "event" else ""}{self._parent!s} {r}'
 
         if self._df:
             return self._parent._parent._parent.df(rs)[tr]
